/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib/gi18n.h>
#include <cairo.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "edecor.h"
#include "ehid.h"
#include "enpathia.h"
#include "keyboard.h"
#include "keys.h"
#include "menu.h"
#include "misc.h"
#include "profile.h"
#include "protocol.h"
#include "unlock.h"
#include "calibration.h"

#define SAMPLES_STAGE	100
#define SAMPLES_START	30
#define SAMPLES_END		90

#define N_STAGES	5

gboolean calibrating = FALSE;

static struct
{
	GtkWidget*	window_calibration;
	GtkWidget*	vbox_calibration;
	GtkWidget*	label_title;
	GtkWidget*	label_message;
	GtkWidget*	image;
}
widgets;

static GdkPixbuf* 	images [3] = { NULL };
static int			indice_imagen_stage [N_STAGES] = { 0, 1, 0, 2, 0 };

static const gchar*	message [N_STAGES] =
{
	N_("Stay in the rest position."),
	N_("Tilt your head to the right."),
	N_("Return to the rest position."),
	N_("Turn your head up."),
	N_("Return to the rest position.")
};

static gint	sample;
static gint	stage;

static void 	
normalize 		(gfloat* v);
static gfloat 	
determinant 	(gfloat* m);
static gfloat 	
cramer 			(gfloat* m, gfloat v1, gfloat v2, gfloat v3, int col);
static guchar 	
scale_byte 		(gfloat f);


	void
load_calibration_window (void)
{
	GtkBuilder*	builder;
	gchar*		data_file;

	/* Load the interface file */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/calibration.glade");
	
	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the calibration interface");

	g_free (data_file);

	/* Get the widgets */
#define GET_WIDGET(widget,name) \
	widgets.widget = GTK_WIDGET (gtk_builder_get_object (builder, name)); \
	if (widgets.widget == NULL) g_warning (_("Widget not found: %s"), name);

	GET_WIDGET (window_calibration, "window_calibration");
	GET_WIDGET (vbox_calibration, "vbox_calibration");
	GET_WIDGET (label_title, "label_title");
	GET_WIDGET (label_message, "label_message");
	GET_WIDGET (image, "image");

#undef GET_WIDGET

	gtk_builder_connect_signals (builder, &widgets);
	g_object_unref (G_OBJECT (builder));

	gtk_window_set_keep_above (GTK_WINDOW (widgets.window_calibration), TRUE);
	hide_pointer (widgets.window_calibration);

	edecor_decorate_window (widgets.window_calibration, widgets.vbox_calibration);

	/* Load the images */
	data_file = get_data_file ("/images/calibration/idle.png");
	images [0] = gdk_pixbuf_new_from_file (data_file, NULL); 
	g_free (data_file);

	data_file = get_data_file ("/images/calibration/right.png");
	images [1] = gdk_pixbuf_new_from_file (data_file, NULL); 
	g_free (data_file);

	data_file = get_data_file ("/images/calibration/up.png");
	images [2] = gdk_pixbuf_new_from_file (data_file, NULL);
	g_free (data_file);
}

	void
calibrate (void)
{
	if (calibrating)
		return;

	if (unlocking) stop_unlock ();

	/* Close the menu */
	if (showing_menu)
		hide_menu ();

	/* Press Escape to close open menus */
	send_command (COMMAND_KEY, KEY_ESCAPE, 0);

	/* Change to raw mode */
	change_mode (MODE_RAW);

	sample = 0;
	stage = 0;

	/* Show the window */
	gtk_window_present (GTK_WINDOW (widgets.window_calibration));

	calibrating = TRUE;
	center_pointer ();
	
#ifdef __MACH__
	gtk_window_set_keep_above (GTK_WINDOW (widgets.window_calibration), TRUE);		
#endif
}

	void
stop_calibration ()
{
	calibrating = FALSE;

	gtk_widget_hide (widgets.window_calibration);
}

	void
calibration_handle_raw_sample (guint a, guint b, guint c)
{
	static gfloat accum_a = 0.0f, accum_b = 0.0f, accum_c = 0.0f;
	static gfloat matrix [9];
	gchar*	text;

	if (!calibrating)
		return;

	sample++;

	/* Set the image and text */
	if (sample == 1)
	{
		text = g_strdup_printf ("<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>",
				gettext (message [stage]));
		gtk_label_set_markup (GTK_LABEL (widgets.label_message), text);
		g_free (text);

		gtk_image_set_from_pixbuf (GTK_IMAGE (widgets.image), images [indice_imagen_stage [stage]]);
	}

	/* Accumulate the value of the sample */
	if (sample > SAMPLES_START && sample <= SAMPLES_END)
	{
		accum_a += a;
		accum_b += b;
		accum_c += c;
	}
	/* When we reach the end of a stage calculate the average and set the matrix */
	else if (sample > SAMPLES_STAGE)
	{
		a = accum_a / (SAMPLES_END - SAMPLES_START);
		b = accum_b / (SAMPLES_END - SAMPLES_START);
		c = accum_c / (SAMPLES_END - SAMPLES_START);

		accum_a = 0.0f;
		accum_b = 0.0f;
		accum_c = 0.0f;

		switch (stage)
		{
			case 0:
				/* Idle position */

				matrix [0] = (gfloat) a - 2048.0;
				matrix [1] = (gfloat) b - 2048.0;
				matrix [2] = (gfloat) c - 2048.0;

				stage = 1;
				sample = 0;
				break;

			case 1:
				/* Right */

				matrix [3] = (gfloat) a - 2048.0;
				matrix [4] = (gfloat) b - 2048.0;
				matrix [5] = (gfloat) c - 2048.0;

				stage = 2;
				sample = 0;
				break;

			case 2:
				stage = 3;
				sample = 0;
				break;

			case 3:
				/* Up */

				matrix [6] = (gfloat) a - 2048.0;
				matrix [7] = (gfloat) b - 2048.0;
				matrix [8] = (gfloat) c - 2048.0;

				stage = 4;
				sample = 0;
				break;

			case 4:
				{
					gfloat det;
					gfloat mat_calib [6];

					/* Solve the system */
					det = determinant (matrix);

					mat_calib [0] = cramer (matrix, 0.0, 1.0, 0.0, 0) / det;
					mat_calib [1] = cramer (matrix, 0.0, 1.0, 0.0, 1) / det;
					mat_calib [2] = cramer (matrix, 0.0, 1.0, 0.0, 2) / det;

					mat_calib [3] = cramer (matrix, 0.0, 0.0, -1.0, 0) / det;
					mat_calib [4] = cramer (matrix, 0.0, 0.0, -1.0, 1) / det;
					mat_calib [5] = cramer (matrix, 0.0, 0.0, -1.0, 2) / det;

					normalize (&(mat_calib [0]));
					normalize (&(mat_calib [3]));

					/* Scale to 0-255 to send it to the device */
					current_profile.calibration.matrix [0][0] = scale_byte (mat_calib [0]);
					current_profile.calibration.matrix [0][1] = scale_byte (mat_calib [1]);
					current_profile.calibration.matrix [0][2] = scale_byte (mat_calib [2]);

					current_profile.calibration.matrix [1][0] = scale_byte (mat_calib [3]);
					current_profile.calibration.matrix [1][1] = scale_byte (mat_calib [4]);
					current_profile.calibration.matrix [1][2] = scale_byte (mat_calib [5]);

					/* Send the configuration */
					send_cfg_calibration ();

					/* Close the window and start moving */
					stop_calibration ();
					change_mode (MODE_MOVEMENT);
				}
				break;
		}
	}

	gtk_widget_queue_draw (widgets.window_calibration);
}

	G_MODULE_EXPORT gboolean
drawingarea_calibration_bar_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t* 	cr;
	gint		width, height;

	cr = gdk_cairo_create (widget->window);

	width	= widget->allocation.width;
	height	= widget->allocation.height;

	edecor_draw_window_background (cr, 0, 0, width, height);
	edecor_draw_progress_bar (cr, 0, 0, width, height,
			(gfloat) sample / (gfloat) (SAMPLES_STAGE - 1));

	cairo_destroy (cr);

	return FALSE;
}

	static void
normalize (gfloat* v)
{
	gfloat n;

	n = sqrt (v [0] * v [0] + v [1] * v [1] + v [2] * v [2]);

	if (n == 0.0)
		return;

	v [0] /= n;
	v [1] /= n;
	v [2] /= n;
}

	static gfloat
determinant (gfloat* m)
{
	gfloat d;

	d = 
		m [0] * m [4] * m [8] +
		m [3] * m [7] * m [2] +
		m [6] * m [1] * m [5] -
		m [6] * m [4] * m [2] -
		m [0] * m [7] * m [5] -
		m [3] * m [1] * m [8];

	return d;		
}

	static gfloat
cramer (gfloat* m, gfloat v1, gfloat v2, gfloat v3, int col)
{
	gfloat m2 [9];

	memcpy (m2, m, 9 * sizeof (gfloat));

	m2 [0 + col] = v1;
	m2 [3 + col] = v2;
	m2 [6 + col] = v3;

	return determinant (m2);
}

	static guchar
scale_byte (gfloat f)
{
	gfloat b;

	b = (f * 128.0) + 128.0;

	if (f < 0.0)
		f = 0.0;
	else if (f > 255.0)
		f = 255.0;

	return (unsigned char) b;
}
