/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "misc.h"
#include "dictionary.h"

typedef struct
{
	gunichar	letter;
	GString*	strings;
}
StringList;

static GSequence*	lists = NULL;

static void
free_string_list	(gpointer data);
static StringList*
find_string_list	(gunichar letter);
static int
strcmp_accents		(gchar* s1, gchar* s2, gint n);


	void
init_dictionary (void)
{
	if (lists != NULL)
		g_sequence_free (lists);

	lists = g_sequence_new (free_string_list);
}

	gint 
load_dictionary (gchar* file)
{
	FILE*		f;
	gchar 		word [50];
	gchar*		word_norm;
	gchar*		word_low;
	gchar*		word_next;
	gchar*		p;
	gunichar	letter;
	gint		line;
	StringList*	string_list;

	/* Open the file */
	f = fopen (file, "r");
	if (!f)
	{
		g_warning ("Cannot open file %s", file);
		return -1;
	}

	/* Read the words */
	line = 0;
	while (1)
	{
		line++;

		/* Get a new word */
		fgets (word, sizeof (word), f);
		if (feof (f))
			break;

		/* Ignore words shorter than 4 letters (including
		 * the carriage return) */
		if (g_utf8_strlen (word, -1) < 4)
			continue;

		/* Normalize */
		word_norm = g_utf8_normalize (word, -1, G_NORMALIZE_DEFAULT);
		if (word_norm == NULL)
		{
			g_warning ("Line %d: %s is not a valid UTF-8 sequence", line, word);
			return -1;
		}

		/* Remove the carriage return character */
		p = word_norm + strlen (word_norm);
		while (p > word_norm)
		{
			if (*p == 13 || *p == 10)
				*p = 0;
			p--;
		}

		/* Convert to lowercase */
		word_low = g_utf8_strdown (word_norm, -1);
		g_free (word_norm);

		/* Remove leading non-alphanumerical characters */
		word_next = word_low;
		while (!g_unichar_isalnum (g_utf8_get_char (word_next)))
			word_next = g_utf8_next_char (word_next);

		/* Find the list or create it */
		letter = g_utf8_get_char (word_low);

		string_list = find_string_list (letter);
		if (string_list == NULL)
		{
			string_list = g_malloc (sizeof (StringList));

			string_list->letter = letter;
			string_list->strings = g_string_new (NULL);

			g_sequence_append (lists, string_list);
		}
			
		/* Append the new word, including the NULL terminator */
		word_next = g_utf8_next_char (word_next);
		g_string_append_len (string_list->strings, word_next, strlen (word_next) + 1);

		g_free (word_low);
	}

	fclose (f);

	return 0;
}

	void
look_up_dictionary (gchar* text, GString** words)
{
	StringList*	list;
	gchar*		text_norm;
	gchar*		text_low;
	gchar*		text_next;
	gunichar	letter;
	gint		len;
	gchar*		str;
	gchar*		str_comp;
	gsize		s;
	gsize		t;

	*words = NULL;

	/* The text must be at least 2 characters long */
	if (g_utf8_strlen (text, -1) < 2)
		return;

	/* Normalize the text and convert it to lowercase */
	text_norm = g_utf8_normalize (text, -1, G_NORMALIZE_DEFAULT);
	if (text_norm == NULL)
	{
		g_warning ("%s is not valid UTF-8 sequence", text);
		return;
	}

	text_low = g_utf8_strdown (text_norm, -1);
	g_free (text_norm);

	/* Find the list */
	letter = g_utf8_get_char (text_low);
	list = find_string_list (letter);
	if (list == NULL)
		return;

	/* Create a new string with the words that match */
	*words = g_string_new (NULL);

	str = list->strings->str;
	s = 0;

	text_next = g_utf8_next_char (text_low);
	len = g_utf8_strlen (text_next, -1);

	while (s < list->strings->len)
	{
		t = strlen (str) + 1;

		/* Compare both strings, considering words with accents as
		 * a special case */
		if (strcmp_accents (text_next, str, len) == 0)
		{
			/* Append the composed version of the word, rather
			 * than the normalized one */
			g_string_append_unichar (*words, letter);

			str_comp = g_utf8_normalize (str, -1, G_NORMALIZE_DEFAULT_COMPOSE);
			g_string_append (*words, str_comp);
			g_free (str_comp);

			g_string_append_c (*words, 0);
		}

		str += t;
		s += t;
	}

	g_free (text_low);
}

	static void
free_string_list (gpointer data)
{
	StringList*	string_list;

	string_list = (StringList*) data;

	g_string_free (string_list->strings, TRUE);
	g_free (string_list);
}

	static StringList*
find_string_list (gunichar letter)
{
	GSequenceIter*	iter;
	StringList*		string_list;

	iter = g_sequence_get_begin_iter (lists);
	while (iter != g_sequence_get_end_iter (lists))
	{
		string_list = (StringList*) g_sequence_get (iter);

		if (string_list->letter == letter)
			return string_list;

		iter = g_sequence_iter_next (iter);
	}

	return NULL;
}

	static int
strcmp_accents (gchar* s1, gchar* s2, gint n)
{
	gunichar	c1;
	gunichar	c2;

	/* Compare up to n characters from s1 */
	while (n > 0)
	{
		if (*s1 == 0 || *s2 == 0)
			return -1;

		/* Get the unichars */
		c1 = g_utf8_get_char (s1);
		c2 = g_utf8_get_char (s2);

		/* If c1 is a non-alphanum character c2 must match, but
		 * if it happens the other way round we just skip c2 */
		if (g_unichar_isalnum (c1) && !g_unichar_isalnum (c2))
		{
			s2 = g_utf8_next_char (s2);
			if (*s2 == 0)
				return -1;
			
			c2 = g_utf8_get_char (s2);
		}
		
		/* Compare them */
		if (c1 != c2)
			return -1;

		/* Move to the next char */
		s1 = g_utf8_next_char (s1);
		s2 = g_utf8_next_char (s2);

		n--;
	}

	return 0;
}
