/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <cairo.h>
#include <dirent.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "edecor.h"
#include "misc.h"
#include "profile.h"
#include "window.h"
#include "users.h"

static struct
{
	GtkWidget*	dialog_users;
	GtkWidget*	vbox_users;
	GtkWidget*	treeview;
	GtkWidget* 	button_use;
	GtkWidget* 	button_new;
	GtkWidget* 	button_remove;
	GtkWidget* 	button_import;
	GtkWidget* 	button_export;
}
widgets;

static gchar* selected_profile_name = NULL;

static void renderer_edited_cb (GtkCellRendererText* cell, gchar* path_string, 
		gchar* new_text, gpointer model);

static void 
init_profile_list	 		(void);
static void 
update_profile_list			(void);
static void 
insert_profile 				(gchar* name);
static void 
read_selected_profile_name 	(gchar** name);

static void
enable_buttons 				(void);


	void
create_users_dialog (void)
{
	GtkBuilder*	builder;
	gchar*		data_file;

	/* Load the interface file */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/users.glade");
	
	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the users interface");

	g_free (data_file);

	/* Get the widgets */
#define GET_WIDGET(widget,name) \
	widgets.widget = GTK_WIDGET (gtk_builder_get_object (builder, name)); \
	if (widgets.widget == NULL) g_warning (_("Widget not found: %s"), name);

	GET_WIDGET (dialog_users, "dialog_users");
	GET_WIDGET (vbox_users, "vbox_users");
	GET_WIDGET (treeview, "treeview");
	GET_WIDGET (button_use, "button_use");
	GET_WIDGET (button_new, "button_new");
	GET_WIDGET (button_remove, "button_remove");
	GET_WIDGET (button_import, "button_import");
	GET_WIDGET (button_export, "button_export");

#undef GET_WIDGET

	gtk_builder_connect_signals (builder, &widgets);
	g_object_unref (G_OBJECT (builder));

	edecor_decorate_window (widgets.dialog_users, widgets.vbox_users);

	init_profile_list ();
}

	void
show_users_dialog (void)
{
	gtk_window_present (GTK_WINDOW (widgets.dialog_users));
}

void import_profile_dialog (void)
{
	gchar*	file;
	gchar*	basename;
	gchar*	config_file;
	gchar	name [300];
	gchar	path [300];
	gchar	text [300];

	config_file = get_config_file ("profiles/");
	strcpy (path, config_file);
	g_free (config_file);

	/* Show a file chooser dialog */
	if (file_chooser (&file, _("Import profile"), TRUE, NULL,
				_("enPathia profile files (*.enp)"), "*.enp"))
	{
		/* Check the filename, if it exists rewriting is not allowed */
		basename = g_path_get_basename (file);

		strcpy (name, basename);
		name [strlen (name) - 4] = 0;
		if (profile_exists (name))
		{
			sprintf (text, _("Profile '%s' already exists"), name);
			dialog_info (_("Perfiles"), text);
			return;
		}

		/* Load the profile */
		strcat (path, basename);
		copy_file (file, path);

		update_profile_list ();

		g_free (basename);
		g_free (file);
	}
}

	void
export_profile_dialog (gchar* name)
{
	gchar*	file;
	gchar*	file_ext;
	gchar	text [100];

	strcpy (text, name);
	strcat (text, ".enp");

	/* Show a file chooser dialog */
	if (file_chooser (&file, _("Export profile"), FALSE, text,
				_("enPathia profile files (*.enp)"), "*.enp"))
	{
		if (g_ascii_strcasecmp (&file [strlen (file) - 4], ".enp") == 0)
			file_ext = g_strdup (file);
		else
		{
			file_ext = g_malloc (strlen (file) + 4);
			strcpy (file_ext, file);
			strcat (file_ext, ".enp");
		}

		export_profile (name, file_ext);

		g_free (file);
		g_free (file_ext);
	}
}

	G_MODULE_EXPORT void
button_use_clicked_cb (GtkWidget *widget, gpointer data)
{
	/* Save current profile */
	save_current_profile ();

	/* Load the new one */
	load_profile (selected_profile_name);
	update_main_window ();
	load_configuration_values ();
	enable_buttons ();
}

	G_MODULE_EXPORT void
button_new_clicked_cb (GtkWidget *widget, gpointer data)
{
	gchar	text [100];
	gint	n;

	n = 1;
	do
	{
		sprintf (text, _("Profile %d"), n);
		n++;
	}
	while (profile_exists (text));

	create_profile (text);
	update_profile_list ();
}

	G_MODULE_EXPORT void
button_remove_clicked_cb (GtkWidget *widget, gpointer data)
{
	gchar text [100];

	sprintf (text, _("Are you sure you want to remove profile '%s'?"),
			selected_profile_name);

	if (dialog_yes_no (_("Profiles"), text))
	{
		remove_profile (selected_profile_name);
		update_profile_list ();
	}
}

	G_MODULE_EXPORT void
button_import_clicked_cb (GtkWidget *widget, gpointer data)
{
	import_profile_dialog ();
}

	G_MODULE_EXPORT void
button_export_clicked_cb (GtkWidget *widget, gpointer data)
{
	export_profile_dialog (selected_profile_name);
}

	G_MODULE_EXPORT void
button_close_clicked_cb (GtkWidget *widget, gpointer data)
{
	gtk_widget_hide (widgets.dialog_users);
}

	G_MODULE_EXPORT void
treeview_cursor_changed_cb (GtkWidget *widget, gpointer data)
{
	/* Selection has changed in the profile list. Read the name of the
	 * selected profile */
	read_selected_profile_name (&selected_profile_name);

	/* If it is the current one disable Use and Remove buttons */
	enable_buttons ();
}

	static void
renderer_edited_cb (GtkCellRendererText* cell, gchar* path_string, 
		gchar* new_text, gpointer model)
{
	gchar text [100];

	/* A profile name has just been edited. If the name has not changed
	 * do nothing */
	if (strcmp (selected_profile_name, new_text) == 0)
		return;

	/* If the profile name exists do not allow it */
	if (profile_exists (new_text)) 
	{
		sprintf (text, _("Profile '%s' already exists"), new_text);
		dialog_info (_("Profiles"), text);
		return;
	}

	/* If it does not, rename the profile */
	rename_profile (selected_profile_name, new_text);
	update_profile_list ();

	update_main_window ();
}

	static void
init_profile_list (void)
{
	GtkCellRenderer*	renderer;
	GtkTreeViewColumn*	column;
	GtkListStore*		store;

	/* Create a model form the list */
	renderer = gtk_cell_renderer_text_new ();
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);

	/* Add a single column */
	column = gtk_tree_view_column_new_with_attributes (
			_("Profiles"), renderer, "text", 0, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (widgets.treeview), column);

	store = gtk_list_store_new (1, G_TYPE_STRING);

	/* Associate it to the widget */
	gtk_tree_view_set_model (GTK_TREE_VIEW (widgets.treeview), GTK_TREE_MODEL (store));

	g_object_unref (store);

	g_signal_connect (renderer,
			"edited", (GCallback) renderer_edited_cb, store);

	/* And fill it up */
	update_profile_list ();
}

	static void
update_profile_list (void)
{
	GtkListStore*	store;
	GDir*			dir;
	const gchar*	file_name;
	gchar*			profiles_dir;
	gchar			name [54];
	gchar*			p;

	/* Clear the list */
	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (widgets.treeview)));
	gtk_list_store_clear (store);

	/* List the profile directory, and add a line for each one */
	profiles_dir = get_config_file ("profiles/");
	dir = g_dir_open (profiles_dir, 0, NULL);
	g_free (profiles_dir);

	if (dir)
	{
		for (;;)
		{
			file_name = g_dir_read_name (dir);
			if (!file_name)
				break;

			if (!strcmp (file_name + strlen (file_name) - 4, ".enp"))
			{
				/* Extract the name from the path */
				strcpy (name, file_name);
				p = name;
				while (*p != '.' && p < name + strlen (name))
					p++;
				*p = 0;

				insert_profile (name);
			}
		}

		g_dir_close (dir);
	}

	/* The current one will have been selected */
	read_selected_profile_name (&selected_profile_name);
	enable_buttons ();
}

	static void
insert_profile (gchar* name)
{
	GtkListStore*		store;
	GtkTreeIter			iter;
	GtkTreeSelection*	selection;

	/* Add a row with the profile name as text */
	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (widgets.treeview)));

	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, 0, name, -1);

	/* If the name belongs to the current profile select it */
	if (!strcmp (name, current_profile_name))
	{
		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widgets.treeview));
		gtk_tree_selection_select_iter (selection, &iter);
	}
}

	static void
read_selected_profile_name (gchar** name)
{
	GtkTreeSelection*	selection;
	GtkTreeModel*		model;
	GtkTreeIter			iter;

	g_free (*name);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widgets.treeview));

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		gtk_tree_model_get (model, &iter, 0, name, -1);
	}
	else
		*name = g_strdup ("");
}

static void enable_buttons (void)
{
	gboolean current_profile_selected;

	current_profile_selected =
		(strcmp (selected_profile_name, current_profile_name) == 0);

	gtk_widget_set_sensitive (widgets.button_use, !current_profile_selected);
	gtk_widget_set_sensitive (widgets.button_remove, !current_profile_selected);
}
